import numpy as np
import matplotlib.pyplot as plt
from scipy.integrate import odeint
plt.close()

k=1.07e-2
Ce0=0.01
Cb0=0.02

def avancement(x,t):
    return k*(Ce0-x)*(Cb0-x)

def euler(f,a,b,y0,pas):
    y=y0
    t=a
    liste_y=[y0]
    liste_t=[a]
    while t+pas<=b:
        y=y+pas*f(y,t)
        liste_y.append(y)
        t=t+pas
        liste_t.append(t)
    return liste_t,liste_y

tmax=10*3600

t,c=euler(avancement,0,tmax,0,1)

ode=odeint(avancement,0,np.array(t))

plt.figure(0)
plt.plot(t,c)
plt.plot(t,ode)
plt.show()

def SeuilDicho(seuil,liste):
    """Recherche le dépassement d'un seuil pour une liste croissante de données numériques
    Entrées: seuil (type 'float') et liste (type 'list')
    Sortie: position dans la liste de la valeur juste avant le dépassement du seuil (type 'int')
    """
    assert (liste[0]-seuil)*(liste[-1]-seuil)<0
    xg,xd=0,len(liste)-1
    yg,yd=liste[0],liste[-1]
    while xd-xg>2:
        xm=(xg+xd)//2
        ym=liste[xm]
        if ym==seuil:
            return xm
        elif (yd-seuil)*(ym-seuil)<0:
            xg,yg=xm,ym
        else:
             xd,yd=xm,ym
    i=xm+1
    return i


position_tdemi=SeuilDicho(0.5*0.01,ode)
tdemi=t[position_tdemi]

h=tdemi//3600
m=tdemi%3600//60
s=tdemi%3600%60

plt.figure(1)
plt.plot(t,c,'b')
plt.plot([tdemi,tdemi],[0,c[position_tdemi]], color ='blue', linewidth=1.5, linestyle="--")
plt.plot([0,tdemi],[c[position_tdemi],c[position_tdemi]], color ='blue', linewidth=1.5, linestyle="--")
plt.xlim(0,tmax)
plt.ylim(0)
plt.xlabel("t en s")
plt.ylabel("Avancement volumique en mol/L")
plt.savefig('sapo2.jpg')
plt.show()



 











